/*
 * camera.h
 *
 *  Created on: 31 mar 2018
 *      Author: mzaleczny
 */

#ifndef SRC_CAMERA_H_
#define SRC_CAMERA_H_

#define CAMERA_MOVEMENT_FORWARD				0
#define CAMERA_MOVEMENT_BACKWARD			1
#define CAMERA_MOVEMENT_LEFT				2
#define CAMERA_MOVEMENT_RIGHT				3
#define CAMERA_MOVEMENT_UP					4
#define CAMERA_MOVEMENT_DOWN				5

#define CAMERA_DEFAULT_YAW					-90.0f
#define CAMERA_DEFAULT_PITCH				0.0f
#define CAMERA_DEFAULT_SPEED_FACTOR			0.005f
#define CAMERA_DEFAULT_MOUSE_SENSITIVITY	0.1f
#define CAMERA_DEFAULT_FOV					45.0f



typedef struct _camera {
	float position[3];
	float front[3];
	float up[3];
	float right[3];
	float world_up[3];

	float yaw;
	float pitch;

	float movement_speed_factor;
	float mouse_sensitivity;
	float fov;
} t_camera;

void init_camera(t_camera *camera, float cam_position[3], float cam_front[3], float world_up[3], float yaw, float pitch,
		float movement_speed_factor, float mouse_sensitivity, float fov);
void update_camera_vectors(t_camera *camera);
void get_look_at_matrix(float lookat_matrix[16], float cam_position[3], float cam_front[3], float up_vector[3]);
void get_camera_look_at_matrix(float lookat_matrix[16], t_camera *camera);

void camera_zoom(t_camera *camera, float y);
void camera_rotate(t_camera *camera, float x_offset, float y_offset, unsigned char constrain_pitch);
void camera_move(t_camera *camera, int direction, float delta_time);

#endif /* SRC_CAMERA_H_ */
