/*
 * engine.h
 *
 *  Created on: 12 paź 2017
 *      Author: mzaleczny
 */

#ifndef SRC_ENGINE_H_
#define SRC_ENGINE_H_

#include <GL/glew.h>
#include <GL/glu.h>

#include <SDL2/SDL.h>
#include <SDL2/SDL_opengl.h>
#include <SDL2/SDL_image.h>
#include <SDL2/SDL_ttf.h>


#define BUFFER_MAIN				0
#define BUFFER_INDICES			1
#define BUFFER_TEXTURE_COORDS	2
#define BUFFER_TEXT				3
#define BUFFERS_COUNT			4

#define PROGRAM_MAIN			0
#define PROGRAM_TEXT			1
#define PROGRAM_TEXTURE			2
#define PROGRAMS_COUNT			3


extern GLuint engine_program[PROGRAMS_COUNT];
extern GLuint engine_vertex_array;
extern GLuint engine_buffers[BUFFERS_COUNT];

extern SDL_Window *g_window;
extern SDL_GLContext g_context;
extern SDL_Surface *image2d;

// Shader's sources
extern char *text_vertex_shader_glsl;
extern char *text_fragment_shader_glsl;
extern GLuint text_vertex_shader;
extern GLuint text_fragment_shader;

extern char *texture_vertex_shader_glsl;
extern char *texture_fragment_shader_glsl;
extern GLuint texture_vertex_shader;
extern GLuint texture_fragment_shader;

extern float projection_matrix[16];
extern float view_matrix[16];
extern float model_matrix[16];
extern float modelview_matrix[16];
extern float normal_matrix[16];
extern float mvp_matrix[16];
extern float aux_matrix1[16];
extern float aux_matrix2[16];

extern int screen_width;
extern int screen_height;
extern float screen_left;
extern float screen_right;
extern float screen_bottom;
extern float screen_top;
extern float screen_near;
extern float screen_far;

unsigned char init_engine();
unsigned char init_engine_with_sdl_window(int screen_w, int screen_h, char *window_caption,
		float f_screen_left, float f_screen_right, float f_screen_bottom, float f_screen_top, float f_screen_near, float f_screen_far);
void cleanup_engine();


#endif /* SRC_ENGINE_H_ */
