#ifndef __MATERIALS_H__
#define __MATERIALS_H__


// stałe określające poszczególne materiały

// materiał domyślny
#define MTL_DEFAULT             0
// mosiądz
#define MTL_BRASS               1
// brąz
#define MTL_BRONZE              2
// polerowany brąz
#define MTL_POLISHED_BRONZE     3
// chrom
#define MTL_CHROME              4
// miedź
#define MTL_COPPER              5
// polerowana miedź
#define MTL_POLISHED_COPPER     6
// złoto
#define MTL_GOLD                7
// polerowane złoto
#define MTL_POLISHED_GOLD       8
// grafit (cyna z ołowiem)
#define MTL_PEWTER              9
// srebro
#define MTL_SILVER              10
// polerowane srebro
#define MTL_POLISHED_SILVER     11
// szmaragd
#define MTL_EMERALD             12
// jadeit
#define MTL_JADE                13
// obsydian (szkło wulkaniczne)
#define MTL_OBSIDIAN            14
// perła
#define MTL_PEARL               15
// rubin
#define MTL_RUBY                16
// turkus
#define MTL_TURQUOISE           17
// jasny biały
#define MTL_BRIGHT_WHITE        18
// mniej jasny biały
#define MTL_LESS_BRIGHT_WHITE   19
// ciepły biały
#define MTL_WARMISH_WHITE       20
// zimny biały
#define MTL_COOLISH_WHITE       21
// czarny plastik
#define MTL_BLACK_PLASTIC       22
// niebieskozielony plastik
#define MTL_CYAN_PLASTIC        23
// zielony plastik
#define MTL_GREEN_PLASTIC       24
// czerwony plastik
#define MTL_RED_PLASTIC         25
// biały plastik
#define MTL_WHITE_PLASTIC       26
// żółty plastik
#define MTL_YELLOW_PLASTIC      27
// czarna guma
#define MTL_BLACK_RUBBER        28
// niebieskozielona guma
#define MTL_CYAN_RUBBER         29
// zielona guma
#define MTL_GREEN_RUBBER        30
// czerwona guma
#define MTL_RED_RUBBER          31
// biała guma
#define MTL_WHITE_RUBBER        32
// żółta guma
#define MTL_YELLOW_RUBBER       33

#define MTL_MATERIALS_COUNT     34



// stałe określające właściwości materiałów
#define MTL_AMBIENT             0
#define MTL_DIFFUSE             1
#define MTL_SPECULAR            2
#define MTL_SHININESS           3


// struktura opisująca parametry materiału
typedef struct _MaterialParameters {
    float ambient[4];               // współczynnik odbicia światła otoczenia; oznaczenie Ma
    float diffuse[4];               // współczynnik odbicia światła rozproszonego; oznaczenie Md
    float specular[4];              // współczynnik odbicia światła zwierciadlanego; oznaczenie Ms
    float shininess;                // wykładnik współczynnika funkcji rozbłysku;
                                    // zakres wartości <0;128>; oznaczenie Msh
    const char *name;
} MaterialParameters;

extern MaterialParameters *materials[MTL_MATERIALS_COUNT];




unsigned char init_materials();
float *get_material(const int material, int material_property);
void cleanup_materials();

#endif
