#ifndef __OBJECT3D_H__
#define __OBJECT3D_H__

#include <GL/glew.h>
#include "memory_utils.h"
#include "texture.h"


#define LIGHT_PER_VERTEX_PROGRAM    0
#define LIGHT_PER_FRAGMENT_PROGRAM  1
#define LIGHT_VECTOR_PROGRAM        2

#define OBJ_KD          0
#define OBJ_KA          1
#define OBJ_KE          2
#define OBJ_KS          3
#define OBJ_NS          4
#define OBJ_KN          5
#define OBJ_PROP_COUNT  6

#define OBJ_OPT_CLAMP  1



typedef struct _object3d_vertex {
    float tangent_vector[3];
    float normal_vector[3];
    float texture_coords[2];
    float coords[3];
} t_object3d_vertex;


typedef struct _object3d_property {
    char         *texture_filename;
    int          opt;
    t_texture    texture_map;

    float color[4];
    float offset[3];
    float scale[3];
} t_object3d_property;

typedef struct _object3d_material {
    char *name;
    t_object3d_property properties[OBJ_PROP_COUNT];
} t_object3d_material;


typedef struct _object3d_polygon {
    unsigned int vertex_indices[3];
} t_object3d_polygon;


typedef struct _object3d_line {
    unsigned int vertex_indices[2];
} t_object3d_line;


typedef struct _object3d_surface {
    int material_index;

    unsigned int polygons_indices_buffer_object;
    unsigned int lines_indices_buffer_object;

    t_mem_buffer polygons;  // stores elements of type:  t_object3d_polygon
    t_mem_buffer lines;       // stores elements of type:  t_object3d_line
} t_object3d_surface;


typedef struct _object3d {
    GLuint vertex_array;
    GLuint vertex_buffer;

    // GLSL vertex attributes locations
    int tangent_loc;
    int normal_loc;
    int tex_coord_loc;
    int position_loc;
    int c_loc[OBJ_PROP_COUNT];
    int o_loc[OBJ_PROP_COUNT];
    int M_loc[OBJ_PROP_COUNT];

    float translate_x;
    float translate_y;
    float translate_z;
    float scale_x;
    float scale_y;
    float scale_z;
    float rotate_x;
    float rotate_y;
    float rotate_z;

    /* variables for use by obj loader: */
    t_mem_buffer materials;// stores elements of type:  t_object3d_material
    t_mem_buffer vertices; // stores elements of type:  t_object3d_vertex
    t_mem_buffer surfaces; // stores elements of type:  t_object3d_surface

    t_mem_buffer cached_vertex_indices;  // stores elements of type:  t_object3d_vec3
    t_mem_buffer cached_texture_indices; // stores elements of type:  t_object3d_vec2
    t_mem_buffer cached_normal_indices;  // stores elements of type:  t_object3d_vec3
    t_mem_buffer cached_iv;              // stores elements of type:  t_object3d_iset
} t_object3d;


t_object3d *object3d_load_from_file(char *filename, unsigned char do_calculation_of_normals, unsigned char init_gl_stuff);
t_object3d *object3d_load_from_obj_file(char *filename, unsigned char do_calculation_of_normals);
void object3d_write_file(char *filename, t_object3d *object);
void object3d_write_obj_file(t_object3d *object, char *object_filename, char *material_filename, int precision);
void object3d_set_transformations(t_object3d *object,
        float translate_x, float translate_y, float translate_z,
        float scale_x, float scale_y, float scale_z,
        float rotate_x, float rotate_y, float rotate_z);
void object3d_render(t_object3d *object);
void object3d_free(t_object3d *object);
void object3d_dump(t_object3d *object);
void object3d_get_bound(t_object3d *object, float *bound_array);

/* ----------------
 * gl stuff
 * ---------------- */
void object3d_set_vertex_locations(t_object3d *object, int tangent_location,
        int normal_location, int tex_coord_location, int position_location);
void object3d_set_property_locations(t_object3d *object, int property_index,
        int c, int o, int M);
/* ----------------
 * end gl stuff
 * ---------------- */

#endif
