#ifndef __PARTICLE_SYSTEM__H__
#define __PARTICLE_SYSTEM__H__

#include <GL/glew.h>

#define POSITION_COMPONENT_COUNT                3
#define COLOR_COMPONENT_COUNT                   3
#define VECTOR_COMPONENT_COUNT                  3
#define PARTICLE_START_TIME_COMPONENT_COUNT     1
#define TOTAL_COMPONENT_COUNT       (POSITION_COMPONENT_COUNT + COLOR_COMPONENT_COUNT + VECTOR_COMPONENT_COUNT + PARTICLE_START_TIME_COMPONENT_COUNT)
#define STRIDE                      TOTAL_COMPONENT_COUNT * sizeof(float)


#define PARTICLE_POSITION_X             0
#define PARTICLE_POSITION_Y             1
#define PARTICLE_POSITION_Z             2
#define PARTICLE_COLOR_R                3
#define PARTICLE_COLOR_G                4
#define PARTICLE_COLOR_B                5
#define PARTICLE_DIRECTION_VECTOR_X     6
#define PARTICLE_DIRECTION_VECTOR_Y     7
#define PARTICLE_DIRECTION_VECTOR_Z     8
#define PARTICLE_START_TIME             9
#define PARTICLE_ITEM_SIZE              10



typedef struct {
    float *particles;
    unsigned int max_particle_count;
    unsigned int current_particle_count;
    // next_particle indeks następnej dodawanej cząsteczki, jeśli osiągnie wartość max_particle_count, to jest ustawiane na 0
    // i cząsteczki o nowych indeksach są nadpisywane
    unsigned int next_particle;

    GLuint vertex_array;
    GLuint vertex_buffer;
} particle_system;

particle_system *particle_system_create(unsigned int max_particle_count);
void particle_system_free(particle_system *ps);
void particle_system_add_particle(particle_system *ps, float pos_x, float pos_y, float pos_z,
                  float color_r, float color_g, float color_b,
                  float dir_x, float dir_y, float dir_z, float start_time);
void particle_system_set_uniforms(GLuint program, float mvp_matrix[16], float elapsed_time, GLuint texture_id);
void particle_system_bind_data(particle_system *ps, GLuint program);
void particle_system_draw(particle_system *ps);




typedef struct {
    float position_x;
    float position_y;
    float position_z;

    float color_r;
    float color_g;
    float color_b;

    float angle_variance;
    float speed_variance;

    float rotation_matrix[16];
    float direction_vector[4];
} particle_emiter;

particle_emiter *particle_emiter_create(float position_x, float position_y, float position_z,
                    float direction_x, float direction_y, float direction_z,
                    float color_r, float color_g, float color_b,
                    float angle_variance, float speed_variance);
void particle_emiter_add_particles(particle_emiter *pe, particle_system *ps, float current_time, unsigned int count);

#endif
