/*
 * sprite2d.h
 *
 *  Created on: 23 wrz 2017
 *      Author: mzaleczny
 */

#ifndef SRC_SPRITE2D_H_
#define SRC_SPRITE2D_H_

#include "texture.h"

#define MAX_SPRITE_STATES	16
#define MAX_STATE_FRAMES_COUNT	16

#define SPRITE_STATE_STAND			0
#define SPRITE_STATE_MOVE_RIGHT		1
#define SPRITE_STATE_MOVE_LEFT		2
#define SPRITE_STATE_RUN_RIGHT		3
#define SPRITE_STATE_RUN_LEFT		4
#define SPRITE_STATE_JUMP_RIGHT		5
#define SPRITE_STATE_JUMP_LEFT		6
#define SPRITE_STATE_SHOOT_RIGHT	7
#define SPRITE_STATE_SHOOT_LEFT		8
#define SPRITE_STATE_SNEAK_RIGHT	9
#define SPRITE_STATE_SNEAK_LEFT		10
#define SPRITE_STATE_ROLL_RIGHT		11
#define SPRITE_STATE_ROLL_LEFT		12


typedef struct _sprite_state_frame_sizes {
	float x; 					// położenie x obrazka sprite'a we współrzędnych tekstury
	float y; 					// położenie y obrazka sprite'a we współrzędnych tekstury
	float w; 					// szerokość obrazka sprite'a we współrzędnych tekstury
	float h; 					// wysokość obrazka sprite'a we współrzędnych tekstury
	unsigned long pos_x; 				// pozycja x obrazka sprite'a w pliku obrazka tekstury
	unsigned long pos_y; 				// pozycja y obrazka sprite'a w pliku obrazka tekstury
	unsigned long size_w; 				// szerokość obrazka sprite'a w pliku obrazka tekstury
	unsigned long size_h; 				// wysokość obrazka sprite'a w pliku obrazka tekstury
	float sprite_w;				// szerokość sprite'a na ekranie
	float sprite_h;				// wysokość sprite'a na ekranie
} t_sprite_state_frame_sizes;


typedef struct _sprite_state {
	unsigned long start_frame;
	unsigned long frames_count;
	t_sprite_state_frame_sizes frame_sizes[MAX_STATE_FRAMES_COUNT]; // rozmiary poszczególnych klatek animacji

	float frame_duration_time;	// czas trwania klatki
	int current_frame;			// bieżąca klatka animacji w klatkach zdefiniowanych w bieżącym stanie sprite'a
	float current_frame_duration;   // czas trwania bieżącej klatki animacji, jeśli ta zmienna przekroczy
									// wartość podaną w atrybucie frame_duration_time, to bieżąca klatka jest
									// zmieniana na następną
	unsigned char loop;    		// czy animacja ma być zapętlona?
} t_sprite_state;


typedef struct _sprite2d {
	float x; // położenie lewego górnego wierzchołka sprite'a na osi x
	float y; // położenie lewego górnego wierzchołka sprite'a na osi y
	float z; // położenie wszystkich wierzchołków na osi z

	float vx;              // prędkość na osi OX
	float vy;              // prędkość gracza w pionie
	float ax;              // przyspieszenie gracza w poziomie
	float ay;              // przyspieszenie gracza w pionie
	float max_vx;
	float min_vx;
	float max_vy;
	float min_vy;

	t_sprite_state sprite_state[MAX_SPRITE_STATES];// tablica poszczególnych stanów sprite'a, których parametry są kopiowane do
									// zmiennych current_frame i current_frame_row struktury _sprite2d
	int current_state;	// numer bieżącego stanu

	t_texturemap *texturemap; // mapa tekstur reprezentujących poszczególne klatki sprite's
	unsigned char free_texturemap; // czy zwalniać mapę tekstur w czasie zwalniania sprite'a;
								   // jeśli sprite był tworzony z przekazanie mapy tekstur, to w tym przypadku
								   // mapa nie powinna być zwalniana

	float gl_sprite_coords[4*3];
	float gl_sprite_texture_coords[4*2];
	float gl_sprite_coords_with_texture[4*3 + 4*2];
} t_sprite2d;


t_sprite2d *create_sprite2d(float x, float y, float z, char *filename_texturemap);
t_sprite2d *create_sprite2d_with_texturemap(float x, float y, float z, t_texturemap *texturemap);
void init_static_sprite(t_sprite2d *sprite, float onscreen_sprite_w, float onscreen_sprite_h, float tex_x, float tex_y, float tex_w, float tex_h);
float *get_sprite_coords(t_sprite2d *sprite);
float *get_sprite_coords_with_texture(t_sprite2d *sprite);
float *get_sprite_texture_coords(t_sprite2d *sprite);
void sprite_update(t_sprite2d *sprite, float dt);
void render_sprite(t_sprite2d *sprite);
void free_sprite2d(t_sprite2d *sprite);

// Poniższa funkcja wczytuje parametry stanów z pliku tekstowego
void init_sprite_states(t_sprite2d *sprite, char *fname_frame_parameters);
void set_sprite_state(t_sprite2d *sprite, int state_index, float ax, float ay);
//void dump_sprite_states(t_sprite2d *sprite);

void sprite_move_horizontal(t_sprite2d *sprite);
void sprite_move_vertical(t_sprite2d *sprite);
unsigned char sprite_can_move_down(t_sprite2d *sprite);

#endif /* SRC_SPRITE2D_H_ */
