/*
 * texture.h
 *
 *  Created on: 30 wrz 2017
 *      Author: mzaleczny
 */

#ifndef SRC_TEXTURE_H_
#define SRC_TEXTURE_H_

#include "SDL2/SDL.h"
#include "SDL2/SDL_image.h"


typedef struct _texture {
	unsigned int texture;    // identyfikator opengl-owy tekstury
	unsigned long original_width;	 // rozmiar załadowanego do tekstury obrazka w pikselach
	unsigned long original_height;
} t_texture;

typedef struct _texturemap {
	t_texture texture;
	unsigned long original_sub_width;	 // rozmiar pojedyńczej pod-tekstury w mapie całej załadowanej tekstury
	unsigned long original_sub_height;
	unsigned long rows;				 // ilość wierszy podtekstur w teksturze-mapie
	unsigned long cols;				 // ilość kolumn podtekstur w teksturze-mapie
} t_texturemap;



unsigned char init_texture_stuff();
void cleanup_texture_stuff();


int load_texture(t_texture *texture, char *filename, int min_filter, int mag_filter, int wrap_s, int wrap_t, unsigned char with_mipmap, unsigned char flip_vertical);
int load_texture_with_processing_image(t_texture *texture, char *filename, int min_filter, int mag_filter, int wrap_s, int wrap_t, unsigned char with_mipmap, unsigned char flip_vertical, int (*process_image)(SDL_Surface* surface));
t_texture *load_texture_from_file(char *filename, int min_filter, int mag_filter, int wrap_s, int wrap_t, unsigned char with_mipmap, unsigned char flip_vertical);
t_texture *load_texture_from_file_with_processing_image(char *filename, int min_filter, int mag_filter, int wrap_s, int wrap_t, unsigned char with_mipmap, unsigned char flip_vertical, int (*process_image)(SDL_Surface* surface));
void free_texture(t_texture *texture);
void render_texture_on_shape(t_texture *texture,
		float *shape_vertices, unsigned int shape_vertices_size,
		unsigned int *shape_indices, unsigned int shape_indices_size,
		float *shape_tex_coords, unsigned int shape_tex_coords_size);
void render_texture_on_shape_with_alpha(t_texture *texture,
		float *shape_vertices, unsigned int shape_vertices_size,
		unsigned int *shape_indices, unsigned int shape_indices_size,
		float *shape_tex_coords, unsigned int shape_tex_coords_size, int set_alpha, float alpha);

t_texturemap *load_texturemap_from_file(char *filename, unsigned long sub_width, unsigned long sub_height, int min_filter, int mag_filter, int wrap_s, int wrap_t, unsigned char with_mipmap, unsigned char flip_vertical);
t_texturemap *load_texturemap_from_file_with_processing_image(char *filename, unsigned long sub_width, unsigned long sub_height, int min_filter, int mag_filter, int wrap_s, int wrap_t, unsigned char with_mipmap, unsigned char flip_vertical, int (*process_image)(SDL_Surface* surface));
void free_texturemap(t_texturemap *texturemap);

// creates and returns cube texture from filenames passed in cube_faces array.
// filenames should be passed in following order:
//char *cube_faces[] = {
//      "skybox/right.jpg",
//      "skybox/left.jpg",
//      "skybox/top.jpg",
//      "skybox/bottom.jpg",
//      "skybox/front.jpg",
//      "skybox/back.jpg"
//};
unsigned int loadCubemap(char *cube_faces[]);


#endif /* SRC_TEXTURE_H_ */
