#version 150

uniform mat4 ProjectionMatrix;
uniform mat4 ModelViewMatrix;
uniform mat3 NormalMatrix;
uniform vec4 LightPosition;

in vec4 vPosition;
in vec2 vTexCoord;
in vec3 vNormal;
in vec3 vTangent;

out vec3 fView;
out vec3 fLight;

out VS_OUT {
    vec3 normal;
} vs_out;

void main() {
    // Tangent space vectors give the columns of the eye-to-tangent transform.
    vec3 N = NormalMatrix * vNormal;
    vec3 T = NormalMatrix * vTangent;
    mat3 M = transpose(mat3(T, cross(N, T), N));

    // Compute the per-fragment attributes.
    fView     =  M * vec3(ModelViewMatrix * vPosition);
    fLight    =  M * vec3(ModelViewMatrix * LightPosition);

    vs_out.normal = vec3(ProjectionMatrix*vec4(N, 0.0));
    gl_Position = ProjectionMatrix * ModelViewMatrix * vPosition;
}
